==> snf_3.3.0_webrelease.tar.gz: snf_3.3.0_webrelease/src/snfdc/spawn.r <==
CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      SUBROUTINE spawn(p,ktape)
**
** spawn process i
**
CJN   modified by J. Neugebauer, Uni Erlangen, 2001
CCH   modified by C. Herrmann, Uni Jena, 04-05/2005 
CCH   (Gaussian and ADF interface)
      INCLUDE 'defs.h'
      INTEGER p,info,mbuf
      INCLUDE 'comm.h'
      INCLUDE 'param.h'  
      INCLUDE 'sadmch.h'
      INTEGER*4 iready,kvers
      INTEGER i,idum,j,k,l,ibuf,speed,newtid,ipe,iprc,q,w,ktape
      INTEGER lsnfdc
C#    INTEGER st(MPI_STATUS_SIZE)
      CHARACTER*40 myname,name
      CHARACTER*255 tmpnode,lognode,newarch,b,snfdc
      character*10 calctype     
      logical udalinp
      IF (p.EQ.0) CALL errex('stale pointer in spawn')

      call calc_type(progno,calcno,calctype) 

      IF (is(p+sadmch_nodf).NE.sadmch_wait) RETURN
      IF (is(p+sadmch_cmd).NE.0) THEN
       is(p+sadmch_nodf)=sadmch_remo
       RETURN
      ENDIF

      CALL flush(jtape)

* individually specified directories?
      CALL sadmch_gname(p,myname)
      CALL llen(myname,l)
      CALL readv('TMPDIR('//myname(1:l)//')',tmpnode,tempdir)
      CALL llen(myname,l)
      CALL readv('LOGDIR('//myname(1:l)//')',lognode,logdir)
      CALL llen(myname,l)
      CALL readv('DCPATH('//myname(1:l)//')',snfdc,dcdir)
      CALL sadmch_spmsg(p,'spawn wait                              ')
      is(p+sadmch_nodf)=sadmch_unf
      WRITE (idbg,*) '@spawn@ spawning snfdc process #',
     * is(p+sadmch_numb),
     * ' on ',myname(1:l)

C#    newtid=is(p+sadmch_tid)

C//SKIPC#

*
*     host already member of virtual machine ?
*

* check if there is a node entry in the virtual machine
      q=pvmmch
20    IF (q.EQ.0) THEN ! no; try to add the node to the virtual machine
       WRITE (idbg,*) '@spawn@ adding ',myname(1:l),' to VM'
       CALL PVMFADDHOST(myname(1:l),info)
       IF (info.LT.0) THEN
        WRITE (jtape,*) myname(1:l),': cannot add node to virtual machine'
        is(p+sadmch_nodf)=sadmch_pvme
        RETURN
       ENDIF
       CALL sadmch_new(pvmmch,myname(1:l),' ')
       is(cmch+sadmch_mxlo)=150
       GOTO 21
      ENDIF
      CALL sadmch_gname(q,name)
      CALL llen(name,k)
      IF (name.EQ.myname) THEN
        WRITE (idbg,*) '@spawn@ OK, already in VM'
        GOTO 21
      ENDIF
      q=is(q+sadmch_next)
      GOTO 20
21    CONTINUE

C//SKIPC%2
      CALL PVMFGETTID(PVMALL,i,newtid)
      WRITE (jtape,*) 'PE',i,' - TID ',newtid

C%    CALL llen(snfdc,lsnfdc)
C%    WRITE (idbg,*) '@spawn@ ',snfdc(1:lsnfdc)
C%    write(*,*) 'snfdc: ',snfdc(1:lsnfdc)
C%    CALL PVMFSPAWN(snfdc(1:lsnfdc),PVMHOST, myname(1:l),1,newtid,info)
CJNC%    CALL PVMFSPAWN('jo',PVMHOST, myname(1:l),1,newtid,info)
C%    WRITE (idbg,*) '@spawn@ return: tid=',newtid,' info=',info
C%    CALL flush(idbg)
C%    IF (info.LT.0.OR.newtid.LT.0) THEN
C%     WRITE (jtape,*) '*Error* pvmfspawn ',myname(1:l)
C%     WRITE (jtape,*) '*Error* newtid=',newtid,' info=',info
C%     CALL sadmch_spmsg(p,'comm problems                           ')
C%     CALL PVMFPERROR('spawn',info)
C%     RETURN
C%    ENDIF

      is(p+sadmch_tid)=newtid
C//END

C@    WRITE (*,*) '@spawn@ spawning process for tid ',newtid
      CALL pputstr('spawn: logdir',logdir,newtid,255,401,info)
      CALL pputstr('spawn: progdir',progdir,newtid,255,402,info)
      CALL pputstr('spawn: arpath',arpath,newtid,255,403,info)
      CALL pputstr('spawn: masterid',masterid,newtid,255,404,info)
      CALL pputstr('spawn: tmpnode',tmpnode,newtid,255,405,info)
      CALL pputstr('spawn: mydir',mydir,newtid,255,406,info)
      CALL pputstr('spawn: masternode',masternode,newtid,40,407,info)
C#    CALL pgetstr('spawn: myname',myname,newtid,40,408,info)
C#    CALL sadmch_sname(p,myname)
C//SKIPC#1
      CALL pputstr('spawn: myname',myname,newtid,40,408,info)
      call rkeywd('control','$usedaltoninput',udalinp)
C//SKIPC&
      call sfile('snf_control',newtid,info)
      if(progno.eq.1) then
        call sfile('control',newtid,info)
      else if (progno.eq.2) then
        call sfile('molecule.mol',newtid,info)
      else if(progno.eq.3 then)
        call sfile('gaussian.com',newtid,info)
      else if(progno.eq.4 then)
        call sfile('adf.in',newtid,info)
      end if
      if(progno.eq.1) then
*----------
* Turbomole
*----------
        call sfile('coord',newtid,info)
        call sfile('basis',newtid,info)
        if (isuhf.eq.0) then
          call sfile('mos',newtid,info)
        else
          call sfile('alpha',newtid,info)
          call sfile('beta',newtid,info)
        end if
        if(index(calctype,'RI').ne.0) ridft=1
        if(ridft.gt.0) then
          call sfile('auxbasis',newtid,info)
        end if
      else if(progno.eq.2) then
*-------
* Dalton
*-------
        call sfile('molecule.dal',newtid,info)
        if(index(calctype,'SOPPA').ne.0) then
          call sfile('molecule.dal.B',newtid,info)
        end if
      else if(progno.eq.3) then
*---------------------------------
* Gaussian: no further files needed
*--------------------------------
        continue
      else if(progno.eq.4) then
*---------------------------------
* ADF: no further files needed
*--------------------------------
        continue
      else
        call errex('program type not implemented')
      end if
C//END
      WRITE (idbg,*) '@spawn@ waiting for answer from ',newtid
      CALL flush(idbg)
C#    w=MPI_COMM_WORLD
C#    CALL MPI_RECV(iready,1,MPI_INTEGER,newtid,407,w,st,info)
C#    CALL mpierr('MPI_RECV mynode','spawnmpi',info)
C#    CALL MPI_RECV(kvers,1,MPI_INTEGER,newtid,408,w,st,info)
C#    CALL mpierr('MPI_RECV kvers','spawnmpi',info)
C//SKIPC#
      CALL PVMFRECV(newtid,0,ibuf)
      CALL PVMFUNPACK(INTEGER4,iready,1,1,info)
      kvers=0
      CALL PVMFUNPACK(INTEGER4,kvers,1,1,info)
C//END
      WRITE (ktape,*) myname,' ',newtid,' ',iready,kvers
      CALL flush(jtape)
      IF (iready.EQ.okok) THEN
       is(p+sadmch_nodf)=sadmch_rdy
      ENDIF
      CALL flush(idbg)
      RETURN
      END

==> snf-4.0.0_webrelease.tar.gz: snf-4.0.0_webrelease/src/snfdc/spawn.r <==
CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      SUBROUTINE spawn(p,ktape)
**
** spawn process i
**
CJN   modified by J. Neugebauer, Uni Erlangen, 2001
CCH   modified by C. Herrmann, Uni Jena, 04-05/2005 
CCH   (Gaussian and ADF interface)
CSL   modified by S. Luber, ETH Zuerich, June 2006
      INCLUDE 'defs.h'
      INTEGER p,info,mbuf
      INCLUDE 'comm.h'
      INCLUDE 'param.h'  
      INCLUDE 'sadmch.h'
      INTEGER*4 iready,kvers
      INTEGER i,idum,j,k,l,ibuf,speed,newtid,ipe,iprc,q,w,ktape
      INTEGER lsnfdc
C#    INTEGER st(MPI_STATUS_SIZE)
      CHARACTER*40 myname,name
      CHARACTER*255 tmpnode,lognode,newarch,b,snfdc
      character*10 calctype     
      logical udalinp
      IF (p.EQ.0) CALL errex('stale pointer in spawn')

      call calc_type(progno,calcno,calctype) 

      IF (is(p+sadmch_nodf).NE.sadmch_wait) RETURN
      IF (is(p+sadmch_cmd).NE.0) THEN
       is(p+sadmch_nodf)=sadmch_remo
       RETURN
      ENDIF

      CALL flush(jtape)

* individually specified directories?
      CALL sadmch_gname(p,myname)
      CALL llen(myname,l)
      CALL readv('TMPDIR('//myname(1:l)//')',tmpnode,tempdir)
      CALL llen(myname,l)
      CALL readv('LOGDIR('//myname(1:l)//')',lognode,logdir)
      CALL llen(myname,l)
      CALL readv('DCPATH('//myname(1:l)//')',snfdc,dcdir)
      CALL sadmch_spmsg(p,'spawn wait                              ')
      is(p+sadmch_nodf)=sadmch_unf
      WRITE (idbg,*) '@spawn@ spawning snfdc process #',
     * is(p+sadmch_numb),
     * ' on ',myname(1:l)

C#    newtid=is(p+sadmch_tid)

C//SKIPC#

*
*     host already member of virtual machine ?
*

* check if there is a node entry in the virtual machine
      q=pvmmch
20    IF (q.EQ.0) THEN ! no; try to add the node to the virtual machine
       WRITE (idbg,*) '@spawn@ adding ',myname(1:l),' to VM'
       CALL PVMFADDHOST(myname(1:l),info)
       IF (info.LT.0) THEN
        WRITE (jtape,*) myname(1:l),': cannot add node to virtual machine'
        is(p+sadmch_nodf)=sadmch_pvme
        RETURN
       ENDIF
       CALL sadmch_new(pvmmch,myname(1:l),' ')
       is(cmch+sadmch_mxlo)=150
       GOTO 21
      ENDIF
      CALL sadmch_gname(q,name)
      CALL llen(name,k)
      IF (name.EQ.myname) THEN
        WRITE (idbg,*) '@spawn@ OK, already in VM'
        GOTO 21
      ENDIF
      q=is(q+sadmch_next)
      GOTO 20
21    CONTINUE

C//SKIPC%2
      CALL PVMFGETTID(PVMALL,i,newtid)
      WRITE (jtape,*) 'PE',i,' - TID ',newtid

C%    CALL llen(snfdc,lsnfdc)
C%    WRITE (idbg,*) '@spawn@ ',snfdc(1:lsnfdc)
C%    write(*,*) 'snfdc: ',snfdc(1:lsnfdc)
C%    CALL PVMFSPAWN(snfdc(1:lsnfdc),PVMHOST, myname(1:l),1,newtid,info)
CJNC%    CALL PVMFSPAWN('jo',PVMHOST, myname(1:l),1,newtid,info)
C%    WRITE (idbg,*) '@spawn@ return: tid=',newtid,' info=',info
C%    CALL flush(idbg)
C%    IF (info.LT.0.OR.newtid.LT.0) THEN
C%     WRITE (jtape,*) '*Error* pvmfspawn ',myname(1:l)
C%     WRITE (jtape,*) '*Error* newtid=',newtid,' info=',info
C%     CALL sadmch_spmsg(p,'comm problems                           ')
C%     CALL PVMFPERROR('spawn',info)
C%     RETURN
C%    ENDIF

      is(p+sadmch_tid)=newtid
C//END

C@    WRITE (*,*) '@spawn@ spawning process for tid ',newtid
      CALL pputstr('spawn: logdir',logdir,newtid,255,401,info)
      CALL pputstr('spawn: progdir',progdir,newtid,255,402,info)
      CALL pputstr('spawn: arpath',arpath,newtid,255,403,info)
      CALL pputstr('spawn: masterid',masterid,newtid,255,404,info)
      CALL pputstr('spawn: tmpnode',tmpnode,newtid,255,405,info)
      CALL pputstr('spawn: mydir',mydir,newtid,255,406,info)
      CALL pputstr('spawn: masternode',masternode,newtid,40,407,info)
C#    CALL pgetstr('spawn: myname',myname,newtid,40,408,info)
C#    CALL sadmch_sname(p,myname)
C//SKIPC#1
      CALL pputstr('spawn: myname',myname,newtid,40,408,info)
      call rkeywd('control','$usedaltoninput',udalinp)
C//SKIPC&
      call sfile('snf_control',newtid,info)
      if(progno.eq.1) then
        call sfile('control',newtid,info)
      else if (progno.eq.2) then
        call sfile('molecule.mol',newtid,info)
      else if(progno.eq.3) then
        call sfile('gaussian.com',newtid,info)
      else if(progno.eq.4) then
        call sfile('adf.in',newtid,info)
      else if(progno.eq.5) then
        call sfile('molpro.in',newtid,info)
      end if
      if(progno.eq.1) then
*----------
* Turbomole
*----------
        call sfile('coord',newtid,info)
        call sfile('basis',newtid,info)
        if (isuhf.eq.0) then
          call sfile('mos',newtid,info)
        else
          call sfile('alpha',newtid,info)
          call sfile('beta',newtid,info)
        end if
        if(index(calctype,'RI').ne.0) ridft=1
        if(ridft.gt.0) then
          call sfile('auxbasis',newtid,info)
        end if
      else if(progno.eq.2) then
*-------
* Dalton
*-------
        call sfile('molecule.dal',newtid,info)
        if(index(calctype,'SOPPA').ne.0) then
          call sfile('molecule.dal.B',newtid,info)
        end if
      else if(progno.eq.3) then
*---------------------------------
* Gaussian: no further files needed
*--------------------------------
        continue
      else if(progno.eq.4) then
*---------------------------------
* ADF: no further files needed
*--------------------------------
        continue
      else if(progno.eq.5) then 
*---------------------------------
*Molpro: no further files needed
*---------------------------------
        continue
      else
        call errex('program type not implemented')
      end if
C//END
      WRITE (idbg,*) '@spawn@ waiting for answer from ',newtid
      CALL flush(idbg)
C#    w=MPI_COMM_WORLD
C#    CALL MPI_RECV(iready,1,MPI_INTEGER,newtid,407,w,st,info)
C#    CALL mpierr('MPI_RECV mynode','spawnmpi',info)
C#    CALL MPI_RECV(kvers,1,MPI_INTEGER,newtid,408,w,st,info)
C#    CALL mpierr('MPI_RECV kvers','spawnmpi',info)
C//SKIPC#
      CALL PVMFRECV(newtid,0,ibuf)
      CALL PVMFUNPACK(INTEGER4,iready,1,1,info)
      kvers=0
      CALL PVMFUNPACK(INTEGER4,kvers,1,1,info)
C//END
      WRITE (ktape,*) myname,' ',newtid,' ',iready,kvers
      CALL flush(jtape)
      IF (iready.EQ.okok) THEN
       is(p+sadmch_nodf)=sadmch_rdy
      ENDIF
      CALL flush(idbg)
      RETURN
      END
